'use strict';

var React = require('react');
var ReactDOM = require('react-dom');
var _ = require('lodash');

var iconClasses = {
    close: 'fa fa-close'
};

module.exports = React.createClass({
    displayName: 'Modal',

    propTypes: {
        closeModalCallback: React.PropTypes.func,
        backgroundClickToClose: React.PropTypes.bool,
        iconClasses: React.PropTypes.object,
        showCloseIcon: React.PropTypes.bool,
        title: React.PropTypes.string
    },

    getDefaultProps: function getDefaultProps() {
        return {
            backgroundClickToClose: true,
            showCloseIcon: true
        };
    },

    getInitialState: function getInitialState() {
        this.iconClasses = _.merge(_.clone(iconClasses), this.props.iconClasses);

        return {};
    },

    componentDidMount: function componentDidMount() {
        this.refs.content.focus();
    },

    componentDidUpdate: function componentDidUpdate() {
        this.refs.content.focus();
    },

    /**
     * Gets markup to display close icon in upper right corner. Only returns markup if
     * the showCloseIcon prop is set.
     * @return {null|ReactElement} Icon markup or null
     */
    getCloseIconMarkup: function getCloseIconMarkup() {
        if (!this.props.showCloseIcon) {
            return null;
        }
        return React.createElement(
            'span',
            { className: 'close', onClick: this.closeModalHandler },
            React.createElement(
                'span',
                { className: 'close-text' },
                'esc to close'
            ),
            ' | ',
            React.createElement('i', { className: this.iconClasses.close })
        );
    },

    render: function render() {
        return React.createElement(
            'div',
            { onClick: this.backgroundClickHandler, id: 'modal-container', 'data-clickcatcher': 'true' },
            React.createElement(
                'div',
                { ref: 'content', className: 'content', tabIndex: '-1', onKeyDown: this.keyDownHandler },
                React.createElement(
                    'div',
                    { className: 'header' },
                    React.createElement(
                        'span',
                        { className: 'title' },
                        this.props.title
                    ),
                    this.getCloseIconMarkup()
                ),
                React.createElement(
                    'div',
                    { className: 'body' },
                    this.props.children
                )
            )
        );
    },

    /**
     * If the key pressed is the escape key and the close icon is being shown, the close modal handler will be called.
     * @param {object} e - The simulated React event.
     */
    keyDownHandler: function keyDownHandler(e) {
        // escape key pressed
        if (e.keyCode === 27 && this.props.showCloseIcon) {
            this.closeModalHandler(e);
        }
    },

    /**
     * Captures any click event outside the modal and calls the close modal handler.
     * @param {Object} e - The simulated React event.
     */
    backgroundClickHandler: function backgroundClickHandler(e) {
        if (this.props.backgroundClickToClose && e.target.getAttribute('data-clickcatcher')) {
            this.closeModalHandler(e);
        }
    },

    /**
     * Triggered when clicking outside the modal, clicking on the close button, and when pressing escape.
     * @param {Object} e - The simulated React event.
     */
    closeModalHandler: function closeModalHandler(e) {
        e.stopPropagation();

        if (typeof this.props.closeModalCallback === 'function') {
            this.props.closeModalCallback();
        } else {
            ReactDOM.unmountComponentAtNode(this.parentNode);
        }
    }
});